@@ -20,7 +20,8 @@ static int recordCallback(const void *inputBuffer, void *outputBuffer,
                           unsigned long framesPerBuffer, const PaStreamCallbackTimeInfo *timeInfo,
                           PaStreamCallbackFlags statusFlags, void *userData) {
     paTestData *data = (paTestData *)userData;
-    SAMPLE *rptr = (SAMPLE *)inputBuffer;
+    unsigned char *rptr = (unsigned char *)inputBuffer;
+    unsigned char *silence_ptr = (unsigned char *)silence_buffer;
     long framesToCalc;
     int finished;
     unsigned long framesLeft = data->maxFrameIndex - data->frameIndex;
@@ -37,14 +38,10 @@ static int recordCallback(const void *inputBuffer, void *outputBuffer,
         finished = paContinue;
     }
 
-    pthread_mutex_lock(&lock);
-
     if (inputBuffer == NULL)
-        write_to_fftw_input_buffers(framesToCalc, silence_buffer, audio);
+        write_to_cava_input_buffers(framesToCalc * 2, silence_ptr, audio);
     else
-        write_to_fftw_input_buffers(framesToCalc, rptr, audio);
-
-    pthread_mutex_unlock(&lock);
+        write_to_cava_input_buffers(framesToCalc * 2, rptr, audio);
 
     data->frameIndex += framesToCalc;
     if (finished == paComplete) {
@@ -125,7 +122,7 @@ void *input_portaudio(void *audiodata) {
     inputParameters.device = deviceNum;
 
     // set parameters
-    data.maxFrameIndex = audio->FFTtreblebufferSize * 1024;
+    data.maxFrameIndex = audio->input_buffer_size * 1024 / 2;
     data.recordedSamples = (SAMPLE *)malloc(2 * data.maxFrameIndex * sizeof(SAMPLE));
     if (data.recordedSamples == NULL) {
         fprintf(stderr, "Error: failure in memory allocation!\n");
@@ -140,10 +137,10 @@ void *input_portaudio(void *audiodata) {
     inputParameters.hostApiSpecificStreamInfo = NULL;
 
     // set it to work
-    err = Pa_OpenStream(&stream, &inputParameters, NULL, audio->rate, audio->FFTtreblebufferSize,
+    err = Pa_OpenStream(&stream, &inputParameters, NULL, audio->rate, audio->input_buffer_size / 2,
                         paClipOff, recordCallback, &data);
     if (err != paNoError) {
-        fprintf(stderr, "Error: failure in opening stream (%x)\n", err);
+        fprintf(stderr, "Error: failure in opening stream (%s)\n", Pa_GetErrorText(err));
         exit(EXIT_FAILURE);
     }
 
@@ -153,7 +150,7 @@ void *input_portaudio(void *audiodata) {
         data.frameIndex = 0;
         err = Pa_StartStream(stream);
         if (err != paNoError) {
-            fprintf(stderr, "Error: failure in starting stream (%x)\n", err);
+            fprintf(stderr, "Error: failure in starting stream (%s)\n", Pa_GetErrorText(err));
             exit(EXIT_FAILURE);
         }
 
@@ -165,7 +162,7 @@ void *input_portaudio(void *audiodata) {
         }
         // check for errors
         if (err < 0) {
-            fprintf(stderr, "Error: failure in recording audio (%x)\n", err);
+            fprintf(stderr, "Error: failure in recording audio (%s)\n", Pa_GetErrorText(err));
             exit(EXIT_FAILURE);
         }
 
@@ -175,7 +172,7 @@ void *input_portaudio(void *audiodata) {
     }
     // close stream
     if ((err = Pa_CloseStream(stream)) != paNoError) {
-        fprintf(stderr, "Error: failure in closing stream (%x)\n", err);
+        fprintf(stderr, "Error: failure in closing stream (%s)\n", Pa_GetErrorText(err));
         exit(EXIT_FAILURE);
     }
 
