@@ -12,6 +12,7 @@ void cb(__attribute__((unused)) pa_context *pulseaudio_context, const pa_server_
 
     // getting default sink name
     struct audio_data *audio = (struct audio_data *)userdata;
+    pthread_mutex_lock(&audio->lock);
     free(audio->source);
     audio->source = malloc(sizeof(char) * 1024);
 
@@ -19,6 +20,7 @@ void cb(__attribute__((unused)) pa_context *pulseaudio_context, const pa_server_
 
     // appending .monitor suufix
     audio->source = strcat(audio->source, ".monitor");
+    pthread_mutex_unlock(&audio->lock);
 
     // quiting mainloop
     pa_context_disconnect(pulseaudio_context);
@@ -48,7 +50,7 @@ void pulseaudio_context_state_callback(pa_context *pulseaudio_context, void *use
         pa_operation_unref(pa_context_get_server_info(pulseaudio_context, cb, userdata));
         break;
     case PA_CONTEXT_FAILED:
-        printf("failed to connect to pulseaudio server\n");
+        fprintf(stderr, "failed to connect to pulseaudio server\n");
         exit(EXIT_FAILURE);
         break;
     case PA_CONTEXT_TERMINATED:
@@ -84,10 +86,11 @@ void getPulseDefaultSink(void *data) {
 
     // starting with one nonblokng iteration in case pulseaudio is not able to run
     if (!(ret = pa_mainloop_iterate(m_pulseaudio_mainloop, 0, &ret))) {
-        printf("Could not open pulseaudio mainloop to "
-               "find default device name: %d\n"
-               "check if pulseaudio is running\n",
-               ret);
+        fprintf(stderr,
+                "Could not open pulseaudio mainloop to "
+                "find default device name: %d\n"
+                "check if pulseaudio is running\n",
+                ret);
 
         exit(EXIT_FAILURE);
     }
@@ -98,20 +101,14 @@ void getPulseDefaultSink(void *data) {
 void *input_pulse(void *data) {
 
     struct audio_data *audio = (struct audio_data *)data;
-    uint16_t frames = audio->FFTtreblebufferSize;
-    int channels = 2;
-    int16_t buf[frames * channels];
+    uint16_t buffer_size = audio->input_buffer_size * audio->format / 8;
+    unsigned char buf[buffer_size];
 
     /* The sample type to use */
     static const pa_sample_spec ss = {.format = PA_SAMPLE_S16LE, .rate = 44100, .channels = 2};
 
-    audio->format = 16;
-
-    const int frag_size = frames * channels * audio->format / 8 *
-                          2; // we double this because of cpu performance issues with pulseaudio
-
     pa_buffer_attr pb = {.maxlength = (uint32_t)-1, // BUFSIZE * 2,
-                         .fragsize = frag_size};
+                         .fragsize = buffer_size};
 
     pa_simple *s = NULL;
     int error;
@@ -123,6 +120,8 @@ void *input_pulse(void *data) {
                 audio->source, pa_strerror(error));
 
         audio->terminate = 1;
+        pthread_exit(NULL);
+        return 0;
     }
 
     while (!audio->terminate) {
@@ -132,9 +131,7 @@ void *input_pulse(void *data) {
             audio->terminate = 1;
         }
 
-        pthread_mutex_lock(&lock);
-        write_to_fftw_input_buffers(frames, buf, data);
-        pthread_mutex_unlock(&lock);
+        write_to_cava_input_buffers(audio->input_buffer_size, buf, data);
     }
 
     pa_simple_free(s);
