@@ -3,11 +3,19 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef _MSC_VER
+#include <fcntl.h>
+#include <io.h>
+#include <windows.h>
+#else
 #include <sys/ioctl.h>
 #include <termios.h>
 #include <unistd.h>
+#endif
 #include <wchar.h>
 
+#define MAX_GRADIENT_COLOR_DEFS 8
+
 wchar_t *frame_buffer;
 wchar_t *barstring[8];
 wchar_t *spacestring;
@@ -19,22 +27,49 @@ int ttybuf_length;
 
 int setecho(int fd, int onoff) {
 
+#ifdef _MSC_VER
+    HANDLE hStdin = GetStdHandle(STD_INPUT_HANDLE);
+    DWORD mode = 0;
+    GetConsoleMode(hStdin, &mode);
+    if (onoff == 0)
+        SetConsoleMode(hStdin, mode & (~ENABLE_ECHO_INPUT));
+    else
+        SetConsoleMode(hStdin, mode & (ENABLE_ECHO_INPUT));
+
+#else
     struct termios t;
 
     if (tcgetattr(fd, &t) == -1)
         return -1;
 
-    if (onoff == 0)
+    if (onoff == 0) {
         t.c_lflag &= ~(ECHO | ECHOE | ECHOK | ECHONL | ICANON);
-    else
+        t.c_cc[VTIME] = 0;
+        t.c_cc[VMIN] = 0;
+    } else {
         t.c_lflag |= (ECHO | ECHOE | ECHOK | ECHONL | ICANON);
+        t.c_cc[VTIME] = 0;
+        t.c_cc[VMIN] = 1;
+    }
 
     if (tcsetattr(fd, TCSANOW, &t) == -1)
         return -1;
-
+#endif
     return 0;
 }
 
+struct colors {
+    short unsigned int rgb[3];
+};
+
+struct colors *gradient_colors;
+
+struct colors parse_color(char *color_string) {
+    struct colors color;
+    sscanf(++color_string, "%02hx%02hx%02hx", &color.rgb[0], &color.rgb[1], &color.rgb[2]);
+    return color;
+}
+
 // general: cleanup
 void free_terminal_noncurses(void) {
     free(frame_buffer);
@@ -45,9 +80,12 @@ void free_terminal_noncurses(void) {
         free(barstring[i]);
         free(ttybarstring[i]);
     }
+    free(gradient_colors);
 }
 
-int init_terminal_noncurses(int tty, int col, int bgcol, int width, int lines, int bar_width) {
+int init_terminal_noncurses(int tty, char *const fg_color_string, char *const bg_color_string,
+                            int col, int bgcol, int gradient, int gradient_count,
+                            char **gradient_color_strings, int width, int lines, int bar_width) {
 
     free_terminal_noncurses();
 
@@ -106,27 +144,85 @@ int init_terminal_noncurses(int tty, int col, int bgcol, int width, int lines, i
     }
 
     col += 30;
+#ifdef _MSC_VER
+    HANDLE hStdOut = NULL;
+    CONSOLE_CURSOR_INFO curInfo;
 
+    hStdOut = GetStdHandle(STD_OUTPUT_HANDLE);
+    GetConsoleCursorInfo(hStdOut, &curInfo);
+    curInfo.bVisible = FALSE;
+    SetConsoleCursorInfo(hStdOut, &curInfo);
+
+    setlocale(LC_ALL, "en_US.utf8");
+    SetConsoleOutputCP(CP_UTF8);
+    system("cls");
+
+#else
+#ifndef __FreeBSD__
     system("setterm -cursor off");
-    system("setterm -blank 0");
+#endif
+    system("clear");
+#endif
 
     // output: reset console
     printf("\033[0m\n");
-    system("clear");
 
-    if (col)
+    if (col == 38) {
+        struct colors fg_color = parse_color(fg_color_string);
+        printf("\033[38;2;%d;%d;%dm", fg_color.rgb[0], fg_color.rgb[1], fg_color.rgb[2]);
+    } else {
         printf("\033[%dm", col); // setting color
+    }
 
-    // printf("\033[1m"); // setting "bright" color mode, looks cooler... I think
+    if (gradient) {
+        struct colors gradient_color_defs[MAX_GRADIENT_COLOR_DEFS];
+        for (int i = 0; i < gradient_count; i++) {
+            gradient_color_defs[i] = parse_color(gradient_color_strings[i]);
+        }
+
+        gradient_colors = (struct colors *)malloc((lines * 2) * sizeof(struct colors));
+
+        int individual_size = lines / (gradient_count - 1);
+
+        float rest = lines / (float)(gradient_count - 1) - individual_size;
+
+        float rest_total = 0;
+        int gradient_lines = 0;
+
+        for (int i = 0; i < gradient_count - 1; i++) {
+            individual_size = lines / (gradient_count - 1);
+            if (rest_total > 1.0) {
+                individual_size++;
+                rest_total = rest_total - 1.0;
+            }
+            for (int n = 0; n < individual_size; n++) {
+                for (int c = 0; c < 3; c++) {
+                    float next_color =
+                        gradient_color_defs[i + 1].rgb[c] - gradient_color_defs[i].rgb[c];
+                    next_color *= n / (float)individual_size;
+                    gradient_colors[gradient_lines].rgb[c] =
+                        gradient_color_defs[i].rgb[c] + next_color;
+                }
+                gradient_lines++;
+            }
+            rest_total = rest_total + rest;
+        }
+        gradient_colors[lines - 1] = gradient_color_defs[gradient_count - 1];
+    }
 
     if (bgcol != 0) {
 
         bgcol += 40;
-        printf("\033[%dm", bgcol);
+
+        if (bgcol == 48) {
+            struct colors bg_color = parse_color(bg_color_string);
+            printf("\033[48;2;%d;%d;%dm", bg_color.rgb[0], bg_color.rgb[1], bg_color.rgb[2]);
+        } else {
+            printf("\033[%dm", bgcol);
+        }
 
         for (int n = lines; n >= 0; n--) {
             for (int i = 0; i < width; i++) {
-
                 printf(" "); // setting backround color
             }
             if (n != 0)
@@ -136,43 +232,54 @@ int init_terminal_noncurses(int tty, int col, int bgcol, int width, int lines, i
         }
         printf("\033[%dA", lines); // moving cursor back up
     }
-
+#ifdef _MSC_VER
+    setecho(1, 0);
+#else
     setecho(STDIN_FILENO, 0);
-
+#endif
     return 0;
 }
 
 void get_terminal_dim_noncurses(int *width, int *lines) {
 
+#ifdef _MSC_VER
+    CONSOLE_SCREEN_BUFFER_INFO csbi;
+
+    GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), &csbi);
+    *width = csbi.srWindow.Right - csbi.srWindow.Left + 1;
+    *lines = csbi.srWindow.Bottom - csbi.srWindow.Top + 1;
+#else
     struct winsize dim;
 
     ioctl(STDOUT_FILENO, TIOCGWINSZ, &dim);
 
     *lines = (int)dim.ws_row;
     *width = (int)dim.ws_col;
-
-    system("clear"); // clearing in case of resieze
+#endif
 }
 
 int draw_terminal_noncurses(int tty, int lines, int width, int number_of_bars, int bar_width,
-                            int bar_spacing, int rest, int bars[256], int previous_frame[256],
-                            int x_axis_info) {
+                            int bar_spacing, int rest, int bars[], int previous_frame[],
+                            int gradient, int x_axis_info) {
 
     int current_cell, prev_cell, same_line, new_line, cx;
 
-    struct winsize dim;
-
     same_line = 0;
     new_line = 0;
     cx = 0;
 
     if (!tty) {
         // output: check if terminal has been resized
-        ioctl(STDOUT_FILENO, TIOCGWINSZ, &dim);
         if (x_axis_info)
             lines++;
-        if ((int)dim.ws_row != (lines) || (int)dim.ws_col != width)
+
+        int new_lines;
+        int new_width;
+        get_terminal_dim_noncurses(&new_width, &new_lines);
+
+        if (new_lines != (lines) || new_width != width)
             return -1;
+
         if (x_axis_info)
             lines--;
     }
@@ -184,6 +291,20 @@ int draw_terminal_noncurses(int tty, int lines, int width, int number_of_bars, i
 
     for (int current_line = lines - 1; current_line >= 0; current_line--) {
 
+        if (gradient) {
+            if (tty) {
+                cx += snprintf(ttyframe_buffer + cx, ttybuf_length - cx, "\033[38;2;%d;%d;%dm",
+                               gradient_colors[current_line].rgb[0],
+                               gradient_colors[current_line].rgb[1],
+                               gradient_colors[current_line].rgb[2]);
+            } else if (!tty) {
+                cx += swprintf(frame_buffer + cx, buf_length - cx, L"\033[38;2;%d;%d;%dm",
+                               gradient_colors[current_line].rgb[0],
+                               gradient_colors[current_line].rgb[1],
+                               gradient_colors[current_line].rgb[2]);
+            }
+        }
+
         int same_bar = 0;
         int center_adjusted = 0;
 
@@ -287,11 +408,26 @@ int draw_terminal_noncurses(int tty, int lines, int width, int number_of_bars, i
 }
 
 void cleanup_terminal_noncurses(void) {
+#ifdef _MSC_VER
+    setecho(1, 1);
+    HANDLE hStdOut = NULL;
+    CONSOLE_CURSOR_INFO curInfo;
+
+    hStdOut = GetStdHandle(STD_OUTPUT_HANDLE);
+    GetConsoleCursorInfo(hStdOut, &curInfo);
+    curInfo.bVisible = TRUE;
+    SetConsoleCursorInfo(hStdOut, &curInfo);
+    system("cls");
+#else
     setecho(STDIN_FILENO, 1);
-    printf("\033[0m\n");
+#ifdef __FreeBSD__
+    system("vidcontrol -f >/dev/null 2>&1");
+#else
     system("setfont  >/dev/null 2>&1");
     system("setfont /usr/share/consolefonts/Lat2-Fixed16.psf.gz  >/dev/null 2>&1");
     system("setterm -cursor on");
-    system("setterm -blank 10");
+#endif
     system("clear");
+#endif
+    printf("\033[0m\n");
 }
